@@ -21,15 +21,23 @@
 */
 
 #include <config.h>
+#include <string.h>
 #include <glib.h>
 #include <gtk/gtkliststore.h>
+#include <gtk/gtkmain.h>
 #include <gtk/gtktreeview.h>
+#include <gtk/gtktreeselection.h>
 #include <gtk/gtkcellrenderertext.h>
+#include <gtk/gtkentry.h>
 #include <gtk/gtklabel.h>
+#include <gtk/gtkmenu.h>
+#include <gtk/gtkmenuitem.h>
 #include <gtk/gtkwindow.h>
 #include <libgnome/gnome-i18n.h>
+#include <libgnomeui/gnome-entry.h>
 #include <glade/glade.h>
 #include <bonobo/bonobo-control.h>
+#include <bonobo/bonobo-property-bag-client.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-moniker-util.h>
 #include <bonobo/bonobo-property-bag.h>
@@ -42,6 +50,7 @@ typedef struct {
 	GNOME_Spell_Dictionary dict;
 	BonoboPropertyBag *pb;
 
+	gchar *add_language;
 	gchar *language;
 	gchar *word;
 
@@ -54,7 +63,12 @@ typedef struct {
 	GtkWidget *button_add;
 	GtkWidget *button_ignore;
 	GtkWidget *button_skip;
+	GtkWidget *button_back;
 
+	GtkWidget *combo_add;
+	GtkWidget *entry_add;
+	GList *abbrevs;
+	GList *langs;
 } SpellControlData;
 
 enum {
@@ -63,7 +77,9 @@ enum {
 	PROP_SPELL_ADD,
 	PROP_SPELL_IGNORE,
 	PROP_SPELL_SKIP,
+	PROP_SPELL_BACK,
 	PROP_SPELL_LANGUAGE,
+	PROP_SPELL_SINGLE,
 } SpellControlProps;
 
 static void
@@ -91,15 +107,15 @@ clicked_replace (GtkWidget *w, SpellControlData *cd)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	gchar *replacement;
+	gchar *replacement, *language;
 
         if (gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW (cd->list_suggestions)),
 					     &model, &iter)) {
 		CORBA_Environment ev;
-                gtk_tree_model_get (model, &iter, 0, &replacement, -1);
+                gtk_tree_model_get (model, &iter, 0, &replacement, 1, &language, -1);
 
 		CORBA_exception_init (&ev);
-		GNOME_Spell_Dictionary_setCorrection (cd->dict, cd->word, replacement, &ev);
+		GNOME_Spell_Dictionary_setCorrection (cd->dict, cd->word, replacement, language, &ev);
 		CORBA_exception_free (&ev);
 
 		bonobo_pbclient_set_string (BONOBO_OBJREF (cd->pb), "replace", replacement, NULL);
@@ -107,17 +123,76 @@ clicked_replace (GtkWidget *w, SpellControlData *cd)
 	}
 }
 
+static void
+abbrevs_langs_clear (SpellControlData *cd)
+{
+	if (cd->abbrevs) {
+		g_list_foreach (cd->abbrevs, (GFunc) g_free, NULL);
+		g_list_free (cd->abbrevs);
+		cd->abbrevs = NULL;
+	}
+	if (cd->langs) {
+		g_list_foreach (cd->langs, (GFunc) g_free, NULL);
+		g_list_free (cd->langs);
+		cd->langs = NULL;
+	}
+}
+
+static void
+set_entry_add (SpellControlData *cd)
+{
+	
+	CORBA_sequence_GNOME_Spell_Language *language_seq;
+	CORBA_Environment   ev;
+	
+	CORBA_exception_init (&ev);
+	language_seq = GNOME_Spell_Dictionary_getLanguages (cd->dict, &ev);
+	if (ev._major == CORBA_NO_EXCEPTION && language_seq) {
+		gint i, n;
+
+		abbrevs_langs_clear (cd);
+		for (i = 0, n = 1; i < language_seq->_length; i++) {
+			if (strstr (cd->language, language_seq->_buffer[i].abbreviation)) {
+				cd->langs = g_list_append (cd->langs, g_strdup (_(language_seq->_buffer[i].name)));
+				cd->abbrevs = g_list_append (cd->abbrevs, g_strdup (language_seq->_buffer[i].abbreviation));
+			}
+		}
+		gtk_combo_set_popdown_strings (GTK_COMBO (cd->combo_add), cd->langs);
+	}
+	CORBA_exception_free (&ev);
+}
+
+static gchar *
+get_abbrev (SpellControlData *cd)
+{
+	GList *l, *a;
+	const gchar *text;
+
+	text = gtk_entry_get_text (GTK_ENTRY (cd->entry_add));
+
+	if (text)
+		for (l = cd->langs, a = cd->abbrevs; l && a; l = l->next, a = a->next)
+			if (!strcmp (text, l->data))
+				return (gchar *) a->data;
+
+	return NULL;
+}
+
 static void
 clicked_add (GtkWidget *w, SpellControlData *cd)
 {
-	CORBA_Environment ev;
+	gchar *abbrev;
 
 	/* printf ("set add\n"); */
-	CORBA_exception_init (&ev);
-	GNOME_Spell_Dictionary_addWordToPersonal (cd->dict, cd->word, &ev);
-	CORBA_exception_free (&ev);
+	abbrev = get_abbrev (cd);
+	if (abbrev) {
+		CORBA_Environment ev;
 
-	bonobo_pbclient_set_boolean (BONOBO_OBJREF (cd->pb), "add", TRUE, NULL);
+		CORBA_exception_init (&ev);
+		GNOME_Spell_Dictionary_addWordToPersonal (cd->dict, cd->word, abbrev, &ev);
+		CORBA_exception_free (&ev);
+		bonobo_pbclient_set_string (BONOBO_OBJREF (cd->pb), "add", abbrev, NULL);
+	}
 }
 
 static void
@@ -139,16 +214,22 @@ clicked_skip (GtkWidget *w, SpellControlData *cd)
 	bonobo_pbclient_set_boolean (BONOBO_OBJREF (cd->pb), "skip", TRUE, NULL);
 }
 
+static void
+clicked_back (GtkWidget *w, SpellControlData *cd)
+{
+	bonobo_pbclient_set_boolean (BONOBO_OBJREF (cd->pb), "back", TRUE, NULL);
+}
+
 static void
 set_word (SpellControlData *cd, gchar *word)
 {
 	GNOME_Spell_StringSeq *seq;
 	CORBA_Environment ev;
-	gchar *str, *str_utf8;
+	gchar *str;
 	gint i;
 
-	str = g_strdup_printf (_("Suggestions for '%s'"), word);
-	gtk_label_set_text (GTK_LABEL (cd->label_word), str);
+	str = g_strdup_printf (_("_Suggestions for '%s'"), word);
+	gtk_label_set_text_with_mnemonic (GTK_LABEL (cd->label_word), str);
 	g_free (str);
 
 	g_free (cd->word);
@@ -160,11 +241,10 @@ set_word (SpellControlData *cd, gchar *word)
 	CORBA_exception_free (&ev);
 	if (seq) {
 		gtk_list_store_clear (cd->store_suggestions);
-		for (i = 0; i < seq->_length; i ++) {
+		for (i = 0; i < seq->_length; i += 2) {
 
 			gtk_list_store_append (cd->store_suggestions, &cd->iter_suggestions);
-			gtk_list_store_set (cd->store_suggestions, &cd->iter_suggestions, 0, seq->_buffer [i], -1);
-
+			gtk_list_store_set (cd->store_suggestions, &cd->iter_suggestions, 0, seq->_buffer [i], 1, seq->_buffer [i+1], -1);
 		}
 		gtk_widget_grab_focus (cd->list_suggestions);
 		gtk_widget_set_sensitive (GTK_WIDGET (cd->button_replace), seq->_length != 0);
@@ -186,6 +266,8 @@ set_language (SpellControlData *cd, gchar *language)
 	CORBA_exception_init (&ev);
 	GNOME_Spell_Dictionary_setLanguage (cd->dict, cd->language, &ev);
 	CORBA_exception_free (&ev);
+
+	set_entry_add (cd);
 }
 
 static void
@@ -204,9 +286,16 @@ control_set_prop (BonoboPropertyBag *bag,
 	case PROP_SPELL_LANGUAGE:
 		set_language (cd, BONOBO_ARG_GET_STRING (arg));
 		break;
+	case PROP_SPELL_SINGLE:
+		if (BONOBO_ARG_GET_BOOLEAN (arg)) {
+			gtk_widget_hide (cd->button_skip);
+			gtk_widget_hide (cd->button_back);
+		}
+		break;
 	case PROP_SPELL_REPLACE:
 	case PROP_SPELL_ADD:
 	case PROP_SPELL_SKIP:
+	case PROP_SPELL_BACK:
 	case PROP_SPELL_IGNORE:
 		break;
 	default:
@@ -221,6 +310,8 @@ control_destroy (GObject *control, gpointer data)
 	SpellControlData *cd;
 
 	cd = (SpellControlData *) data;
+	/* printf ("release spell control dict\n"); */
+	bonobo_object_release_unref (cd->dict, NULL);
 
 	g_free (cd);
 }
@@ -236,7 +327,7 @@ spell_control_construct (BonoboControl *control, GtkWidget *table, GladeXML *xml
 
 	cd->label_word = glade_xml_get_widget (xml, "label_word");
 	cd->list_suggestions = glade_xml_get_widget (xml, "list_suggestions");
-	cd->store_suggestions = gtk_list_store_new (1, G_TYPE_STRING);
+	cd->store_suggestions = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (cd->list_suggestions), GTK_TREE_MODEL (cd->store_suggestions));
 	
 	gtk_tree_view_append_column (GTK_TREE_VIEW (cd->list_suggestions),
@@ -251,22 +342,29 @@ spell_control_construct (BonoboControl *control, GtkWidget *table, GladeXML *xml
 	cd->button_add     = glade_xml_get_widget (xml, "button_add");
 	cd->button_ignore  = glade_xml_get_widget (xml, "button_ignore");
 	cd->button_skip    = glade_xml_get_widget (xml, "button_skip");
+	cd->button_back    = glade_xml_get_widget (xml, "button_back");
+	cd->combo_add      = glade_xml_get_widget (xml, "combo_add");
+	cd->entry_add      = glade_xml_get_widget (xml, "entry_add");
 
 	g_signal_connect (cd->button_replace, "clicked", G_CALLBACK (clicked_replace), cd);
 	g_signal_connect (cd->button_add, "clicked", G_CALLBACK (clicked_add), cd);
 	g_signal_connect (cd->button_ignore, "clicked", G_CALLBACK (clicked_ignore), cd);
 	g_signal_connect (cd->button_skip, "clicked", G_CALLBACK (clicked_skip), cd);
+	g_signal_connect (cd->button_back, "clicked", G_CALLBACK (clicked_back), cd);
 
 	g_signal_connect (control, "destroy", G_CALLBACK (control_destroy), cd);
 
 	/* PropertyBag */
 	cd->pb = bonobo_property_bag_new (control_get_prop, control_set_prop, cd);
 	bonobo_control_set_properties (control, BONOBO_OBJREF (cd->pb), NULL);
+	bonobo_object_unref (BONOBO_OBJECT (cd->pb));
 
 	bonobo_property_bag_add (cd->pb, "word", PROP_SPELL_WORD, BONOBO_ARG_STRING, NULL,
 				 "checked word",  BONOBO_PROPERTY_WRITEABLE);
 	bonobo_property_bag_add (cd->pb, "language", PROP_SPELL_LANGUAGE, BONOBO_ARG_STRING, NULL,
 				 "dictionary language", BONOBO_PROPERTY_WRITEABLE);
+	bonobo_property_bag_add (cd->pb, "single", PROP_SPELL_SINGLE, BONOBO_ARG_BOOLEAN, NULL,
+				 "check single word", BONOBO_PROPERTY_WRITEABLE);
 
 	def = bonobo_arg_new (BONOBO_ARG_STRING);
 	BONOBO_ARG_SET_STRING (def, "replace default value");
@@ -274,9 +372,9 @@ spell_control_construct (BonoboControl *control, GtkWidget *table, GladeXML *xml
 				 "replacement to replace word",  BONOBO_PROPERTY_READABLE);
 	CORBA_free (def);
 
-	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
-	BONOBO_ARG_SET_BOOLEAN (def, TRUE);
-	bonobo_property_bag_add (cd->pb, "add", PROP_SPELL_REPLACE, BONOBO_ARG_BOOLEAN, def,
+	def = bonobo_arg_new (BONOBO_ARG_STRING);
+	BONOBO_ARG_SET_STRING (def, "");
+	bonobo_property_bag_add (cd->pb, "add", PROP_SPELL_REPLACE, BONOBO_ARG_STRING, def,
 				 "add word to dictionary",  BONOBO_PROPERTY_READABLE);
 	CORBA_free (def);
 
@@ -292,9 +390,15 @@ spell_control_construct (BonoboControl *control, GtkWidget *table, GladeXML *xml
 				 "skip this word",  BONOBO_PROPERTY_READABLE);
 	CORBA_free (def);
 
+	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
+	BONOBO_ARG_SET_BOOLEAN (def, TRUE);
+	bonobo_property_bag_add (cd->pb, "back", PROP_SPELL_REPLACE, BONOBO_ARG_BOOLEAN, def,
+				 "back to prev incorrect word",  BONOBO_PROPERTY_READABLE);
+	CORBA_free (def);
+
 	cd->dict = bonobo_get_object ("OAFIID:GNOME_Spell_Dictionary:" API_VERSION, "GNOME/Spell/Dictionary", NULL);
 
-	set_language (cd, "en");
+	/* set_language (cd, "en_us"); */
 }
 
 BonoboObject *
@@ -311,7 +415,7 @@ gnome_spell_control_new ()
 
 	control = bonobo_control_new (table);
 
-	if (control){
+	if (control) {
 		spell_control_construct (control, table, xml);
 		return BONOBO_OBJECT (control);
 	} else {
