@@ -23,13 +23,12 @@
 #ifndef SPELL_CHECKER_CONTROL_H_
 #define SPELL_CHECKER_CONTROL_H_
 
-#include <libgnome/gnome-defs.h>
-#include <bonobo/bonobo-object.h>
+G_BEGIN_DECLS
 
-BEGIN_GNOME_DECLS
+#include <bonobo/bonobo-object.h>
 
 BonoboObject *gnome_spell_control_new  ();
 
-END_GNOME_DECLS
+G_END_DECLS
 
 #endif
