@@ -143,12 +143,14 @@ release_engines (GNOMESpellDictionary *dict)
 	dict->changed = TRUE;
 }
 
-#define KNOWN_LANGUAGES 25
+#define KNOWN_LANGUAGES 42
 static gchar *known_languages [KNOWN_LANGUAGES*2 + 1] = {
 	"br", N_("Breton"),
 	"ca", N_("Catalan"),
 	"cs", N_("Czech"),
+	"cy", N_("Welsh"),
 	"da", N_("Danish"),
+	"de-AT", N_("German (Austria)"),
 	"de-DE", N_("German (Germany)"),
 	"de-CH", N_("German (Swiss)"),
 	"el", N_("Greek"),
@@ -157,19 +159,34 @@ static gchar *known_languages [KNOWN_LANGUAGES*2 + 1] = {
 	"en-CA", N_("English (Canadian)"),
 	"eo", N_("Esperanto"),
 	"es", N_("Spanish"),
+	"fi", N_("Finnish"),
 	"fo", N_("Faroese"),
 	"fr-FR", N_("French (France)"),
 	"fr-CH", N_("French (Swiss)"),
+	"ga", N_("Irish"),
+	"gl", N_("Galician"),
+	"he", N_("Hebrew"),
+	"hu", N_("Hungarian"),
 	"it", N_("Italian"),
+	"is", N_("Icelandic"),
+	"la", N_("Latin"),
+	"lt", N_("Lithuanian"),
+	"nb-NO", N_("Norwegian (Bokmal)"),
 	"nl", N_("Dutch"),
+	"nn-NO", N_("Norwegian (Nyorsk)"),
 	"no", N_("Norwegian"),
 	"pl", N_("Polish"),
 	"pt-PT", N_("Portuguese (Portugal)"),
 	"pt-BR", N_("Portuguese (Brazilian)"),
+	"ro", N_("Romanian"),
 	"ru", N_("Russian"),
+	"sc", N_("Sardinian"),
 	"sk", N_("Slovak"),
+	"sl", N_("Slovenian"),
 	"sv", N_("Swedish"),
+	"tr", N_("Turkish"),
 	"uk", N_("Ukrainian"),
+	"yi", N_("Yiddish"),
 	NULL
 };
 
@@ -300,7 +317,12 @@ impl_gnome_spell_dictionary_set_language (PortableServer_Servant  servant,
 	gchar *one_language;
 	gint len;
 
-	g_assert (dict);
+#ifndef G_DISABLE_CHECKS
+	g_return_if_fail (dict);
+#else
+	if (!dict)
+		return;
+#endif
 	if (!language)
 		language = "";
 
@@ -357,7 +379,7 @@ update_engine (SpellEngine *se, CORBA_Environment * ev)
 static void
 update_engines (GNOMESpellDictionary *dict, CORBA_Environment * ev)
 {
-	g_assert (IS_GNOME_SPELL_DICTIONARY (dict));
+	g_return_if_fail (IS_GNOME_SPELL_DICTIONARY (dict));
 
 	if (dict->changed) {
 		GSList *l;
@@ -376,8 +398,12 @@ engine_check_word (SpellEngine *se, const gchar *word, CORBA_Environment *ev)
 	CORBA_boolean result = CORBA_TRUE;
 	gint aspell_result;
 
+#ifndef G_DISABLE_CHECKS
 	g_return_val_if_fail (se->speller, CORBA_TRUE);
-
+#else
+	if (!se->speller)
+		return CORBA_TRUE;
+#endif
 	aspell_result = aspell_speller_check (se->speller, word, strlen (word));
 	if (aspell_result == 0)
 		result = CORBA_FALSE;
@@ -397,8 +423,12 @@ impl_gnome_spell_dictionary_check_word (PortableServer_Servant servant, const CO
 	GSList *l;
 	gboolean valid_speller = FALSE;
 
+#ifndef G_DISABLE_CHECKS
 	g_return_val_if_fail (word, result);
-
+#else
+	if (!word)
+		return result;
+#endif
 	if (!strcmp (word, "Ximian"))
 		return CORBA_TRUE;
 
@@ -425,8 +455,12 @@ impl_gnome_spell_dictionary_add_word_to_session (PortableServer_Servant servant,
 	GNOMESpellDictionary *dict = GNOME_SPELL_DICTIONARY (bonobo_object_from_servant (servant));
 	GSList *l;
 
+#ifndef G_DISABLE_CHECKS
 	g_return_if_fail (word);
-
+#else
+	if (!word)
+		return;
+#endif
 	update_engines (dict, ev);
 	DICT_DEBUG (printf ("Dictionary add_word_to_session: %s\n", word));
 	for (l = dict->engines; l; l = l->next) {
@@ -442,8 +476,12 @@ impl_gnome_spell_dictionary_add_word_to_personal (PortableServer_Servant servant
 	GNOMESpellDictionary *dict = GNOME_SPELL_DICTIONARY (bonobo_object_from_servant (servant));
 	SpellEngine *se;
 
-	g_return_if_fail (word);
-
+#ifndef G_DISABLE_CHECKS
+	g_return_if_fail (word && language);
+#else
+	if (!word || !language)
+		return;
+#endif
 	update_engines (dict, ev);
 	DICT_DEBUG (printf ("Dictionary add_word_to_personal: %s (%s)\n", word, language));
 	se = (SpellEngine *) g_hash_table_lookup (dict->languages, language);
@@ -462,9 +500,12 @@ impl_gnome_spell_dictionary_set_correction (PortableServer_Servant servant,
 	GNOMESpellDictionary *dict = GNOME_SPELL_DICTIONARY (bonobo_object_from_servant (servant));
 	SpellEngine *se;
 
-	g_return_if_fail (word);
-	g_return_if_fail (replacement);
-
+#ifndef G_DISABLE_CHECKS
+	g_return_if_fail (word && replacement);
+#else
+	if (!word || !replacement)
+		return;
+#endif
 	update_engines (dict, ev);
 	DICT_DEBUG (printf ("Dictionary correction: %s <-- %s\n", word, replacement));
 	se = (SpellEngine *) g_hash_table_lookup (dict->languages, language);
@@ -488,8 +529,12 @@ impl_gnome_spell_dictionary_get_suggestions (PortableServer_Servant servant,
 	GSList *l, *suggestion_list = NULL;
 	gint i, len, pos;
 
+#ifndef G_DISABLE_CHECKS
 	g_return_val_if_fail (word, NULL);
-
+#else
+	if (!word)
+		return NULL;
+#endif
 	DICT_DEBUG (printf ("Dictionary correction: %s\n", word));
 	update_engines (dict, ev);
 
