@@ -23,59 +23,30 @@
 
 #include <config.h>
 #include <gnome.h>
-#include <liboaf/liboaf.h>
 #include <bonobo.h>
 #include "Spell.h"
 
-static void
-init_bonobo (int argc, char *argv [])
-{
-	CORBA_ORB orb;
-
-        gnome_init_with_popt_table (
-		"test-spell", VERSION,
-		argc, argv,
-		oaf_popt_options, 0, NULL); 
-
-	orb = oaf_init (argc, argv);
-
-	if (!bonobo_init (orb, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL))
-		g_error (_("I could not initialize Bonobo"));
-
-	bonobo_activate ();
-}
-
-static gboolean
-check_obj (BonoboObjectClient *obj, gchar *id)
-{
-	if (!obj)
-		printf ("Could not create an instance of the %s component", id);
-	return obj == NULL;
-}
-
 int
 main (int argc, char *argv [])
 {
-	BonoboObjectClient *checker_client, *dictionary_client;
 	GNOME_Spell_Dictionary en;
 	GNOME_Spell_StringSeq *seq;
 	CORBA_Environment   ev;
-	char *dictionary_id;
+	CORBA_sequence_GNOME_Spell_Language *language_seq;
 	gint i;
 
-	init_bonobo (argc, argv);
+	if (!bonobo_ui_init ("test-spell", VERSION, &argc, argv))
+		g_error (_("I could not initialize Bonobo"));
+	bonobo_activate ();
 
-	dictionary_id = "OAFIID:GNOME_Spell_Dictionary:" API_VERSION;
-	dictionary_client = bonobo_object_activate (dictionary_id, 0);
+	en = bonobo_get_object ("OAFIID:GNOME_Spell_Dictionary:" API_VERSION, "GNOME/Spell/Dictionary", 0);
 
-	if (check_obj (dictionary_client, dictionary_id))
+	if (en == CORBA_OBJECT_NIL) {
+		g_error ("Could not create an instance of the spell component");
 		return 1;
-	CORBA_exception_init (&ev);
+	}
 
-	/*
-	 * Get the CORBA Object reference from the BonoboObjectClient
-	 */
-	en      = bonobo_object_corba_objref (BONOBO_OBJECT (dictionary_client));
+	CORBA_exception_init (&ev);
 
 	/*
 	 * test dictionary
@@ -101,9 +72,16 @@ main (int argc, char *argv [])
 	printf ("check: %s --> %d\n",
 		"helo",
 		GNOME_Spell_Dictionary_checkWord (en, "helo", &ev));
+	language_seq = GNOME_Spell_Dictionary_getLanguages (en, &ev);
+	if (language_seq) {
+		printf ("Languages:\n");
+		for (i = 0; i < language_seq->_length; i++)
+			printf ("\t%s\n", language_seq->_buffer[i].name);
+	}
+
 	CORBA_exception_free (&ev);
 
-	bonobo_object_unref (BONOBO_OBJECT (dictionary_client));
-	
+	bonobo_object_release_unref (en, NULL);
+
 	return 0;
 }
